# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import GridUtils
from Game import GameState

# returns positions flipped by player pnum by moving at pos. Called by Dominion module.
def flipped_positions_for_move(grid, pnum, pos):
    flip = []
    # look in each direction from empty cell
    for d in GridUtils.direction_array(grid.ndim):
        dirflip = []
        # go in same direction until we hit an empty cell, our cell, or the edge
        for nc, val in GridUtils.enumerate_values_with_direction(grid, pos, d):
            if val<=0:
                # empty cell or wall, no flip
                break
            if val==pnum:
                # our cell, line is terminated correctly
                flip.extend(dirflip)
                break
            else:
                # opponent cell, add to potential flip and keep going
                dirflip.append(nc)
    return flip

# returns generator with values (move coord, flipped coords)
def valid_moves_for_player(grid, pnum):
    for pc in GridUtils.enumerate_coords_with_value(grid, 0):
        # move must cause flips in order to be valid
        flip = flipped_positions_for_move(grid, pnum, pc)
        if flip:
            yield (pc, flip)


class ReversiGameState(GameState):
    def successors(self):
        found = False
        for c,flip in valid_moves_for_player(self.grid, self.current_player):
            found = True
            nextgrid = self.grid.copy()
            nextgrid[c] = self.current_player
            for fc in flip:
                nextgrid[fc] = self.current_player
            yield ( (c,), self.next_state_with_grid(nextgrid) )
        # if no moves found, pass is legal
        if not found:
            yield ( (), self.next_state_with_grid(self.grid.copy()))
                
    def is_game_over(self):
        # game over if neither player can move
        # double "any" because the valid_moves generator object is itself true, we need to look at its contents
        return not any(any(valid_moves_for_player(self.grid, pnum)) for pnum in xrange(1, self.num_players+1))
    
